#ifndef _replay_h
#define _replay_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "team.h"
#include "teamplayer.h"
#include "ball.h"
#include "goal.h"
#include "camera.h"

#include "../gui/videocontroller.h"
#include "../etc/cursor.h"

#include <SDL/SDL.h>

#define FRAMES_TO_REPLAY 250  /**< Quantity of frames to replay */

/*! The replay data has all positions and angles
 * of all players, ball, etc on a frame. */
class replayData
{
   public:
      float teamAPositions[11][2];  /**< Positions of teamA players */
      float teamAAngles[11];        /**< Angles of teamA players */
      
      float teamBPositions[11][2];  /**< Positions of teamB players */
      float teamBAngles[11];        /**< Angles of teamB players */
   
      float ballPosition[3];        /**< Position of the ball */
      float ballAngle[3];           /**< Angle of the ball */
};

/*! The replay class saves FRAMES_TO_REPLAY frames and replay them at
 * any time, with a simple video controller. */
class replay
{
   public:

      /*! Constructor
       * \param g -> pointer to goal model
       * \param cam -> pointer to the camera
       * \param pfield -> field used */
      replay(goal *g, camera *cam, field* pfield);

      /*! Destructor */
      ~replay();

      /*! Define team A 
       * \param t -> pointer to the team A */
      void setTeamA(team *t);

      /*! Define team B 
       * \param t -> pointer to the team B */
      void setTeamB(team *t);

      /*! Define ball
       * \param b -> pointer to the current ball */
      void setBall(ball *b);

      /*! Clear all frames current stored on the replay */
      void clear();

      /*! Update all data relative to the frame */
      void updateData();

      /*! Get data for a desired frame
       * \param frame -> frame number to get
       * \return -> data of the desired frame */
      replayData getData(int frame);

      /*! Render the replay (with controllers, etc)
       * \param cursors -> game mouse cursors */
      void renderReplay(cursor* cursors);
   
   protected:

      /*! Verify the replay IO */
      void replayIO();

      /*! Render a replay frame to the screen
       * \param cursors -> mouse cursors
       * \param frame -> number of the frame to render */
      void renderFrame(int frame, cursor* cursors);

      /*! Define the little R texture */
      void defineReplayTexture();

      /*! Render the little R Texture */
      void renderReplayTexture();

      team *teamA;                        /**< Pointer to teamA */
      team *teamB;                        /**< Pointer to teamB */
      ball *gameBall;                     /**< Pointer to the game ball */
      goal* goals;                        /**< The field Goals */
      camera* gameCamera;                 /**< The camera used */
      field* gameField;                   /**< The field used */

      Uint8 *keys;                        /**< current SDL key state */
      int mouseX,                         /**< current mouse X position */
          mouseY;                         /**< current mouse Y position */
      Uint8 mButton;                      /**< current mouse button state */

      GLuint replayTexture;               /**< the little R texture */
      float replayTextureScale;           /**< current little R scale factor */
      float replayTextureDelta;           /**< current little R delta */

      videoController* controller;        /**< The Controller */

      bool exit;                          /**< Flux Controll */
      float delta;                        /**< the variation for frame change */
      float position;                     /**< current position for frame */
      float curSpeed;                     /**< current frame change speed */
      
      int start,                          /**< start frame on the vector */
          currentFrame,                   /**< current frame on the vector */
          end;                            /**< end frame on the vector */
      bool replaying;                     /**< true if is rendering replay */
      replayData data[FRAMES_TO_REPLAY];  /**< The replay Vector */
   
};


#endif
