#ifndef _rules_h
#define _rules_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

//#define RULE_TYPE_BALL_FREE  0
#define RULE_TYPE_BALL_12    1
//#define RULE_TYPE_BALL_3     2
//#define RULE_TYPE_DISK_1     3

#define RULE_TIME_5          5
#define RULE_TIME_10        10
#define RULE_TIME_15        15

#define RULE_STATE_NORMAL              0
#define RULE_STATE_MIDDLE              1
#define RULE_STATE_CORNER_KICK         2
#define RULE_STATE_FREE_KICK           3
#define RULE_STATE_THROW_IN            4
#define RULE_STATE_GOAL_KICK           5
#define RULE_STATE_PENALTY_KICK        6

#include <libintl.h>
#include "team.h"
#include "ball.h"

/*! The rules class is the controller of the game rules. By it 
 *  fouls, next player to play, goals, and all rules are defined
 *  based on the active rule system. */
class rules
{
   public:
      /*! Constructor */
      rules();
      /*! Destructor */
      ~rules();

      /*! Define the current game type
       * \param type -> integer constant with the game type */
      void setGameType(int type);
      /*! Define the length of each half of the match
       * \param minutes -> number of minutes per half */
      void setMinutesPerHalf(int minutes);

      /*! Verify if we are at the first half
       * \return -> true if at the first half. False if at the second. */
      bool isFirstHalf();

      /*! Set the team A pointer
       * \param t -> pointer to the teamA */
      void setTeamA(team* t);

      /*! Set the team B pointer
       * \param t -> pointer to the teamB */
      void setTeamB(team* t);

      /*! Define the used ball
       * \param b -> pointer to the in use ball */
      void setBall(ball* b);

      /*! Set Current In Use Disk 
       * \note Usually called before a shoot or pass 
       * \param disk -> pointer to the current selected teamPlayer
       * \return -> true if can use the disk, false otherwise. */
      bool setDisk(teamPlayer* disk);

      /*! Get the current global remaining touches
       * \return current global remaining touches */
      int getRemainingTouches();
   
      /*! Get the remaining touches for a disk
       * \return current remaining touches for the disk */
      int getRemainingTouches(teamPlayer* disk);

      /*! Init a turn of play
       * \return -> pointer to the current team to act */
      team* newTurn();

      /*! Get the current rules state value
       * \return -> rules state constant. */
      int getState();

      /*! Get the current active team.
       * \return -> pointer to the team acting */
      team* getActiveTeam();
   
      /*! Get the current upper team.
       * \return -> pointer to the team that is at upper field side */
      team* getUpperTeam();

      /*! Tells the opponent that will try a goal shoot now */
      void prepareToShoot();

      /*! Tell the rules that a disk collide another
       * \param diskPlayer1 -> team of the first disk
       * \param diskPlayer2 -> team of the second disk
       * \param x -> X coordinate where the collision occurs
       * \param z -> Z coordinate where the collision occurs */
      void diskCollideDisk(team* diskPlayer1, team* diskPlayer2, 
                           float x, float z);

      /*! Tell the rules that the ball collide with a disk of the player
       * \param player -> team of the disk the ball contacts */
      void ballCollideDisk(team* player);

      /*! Tells that the ball left to one field side
       * \param x -> X coordinate where the exit occurs
       * \param z -> Z coordinate where the exit occurs */
      void ballExitAtSide(float x, float z);

      /*! Tells that the ball left to one field byline
       * \param upper -> true if upper line, false if botton
       * \param z -> Z position where the ball exits */
      void ballExitAtByline(bool upper, float z);

      /*! Tells that the ball enter the goal
       * \param upper -> true if the upper goal, false if botton */
      void ballEnterGoal(bool upper);

      /*! When ball stop move, call this function one time
       * to define the internal state of the rules system. */
      void ballAtFinalPosition();

      /*! Start the Half Now
       * \note -> called two times per game. */
      void startHalf();

      /*! Update the Clock of the current half.
       * \return -> true if the half is over. */
      bool updateClock();

      /*! Clear internal states */
      void clear();

      /*! Set teams and ball positions based on current rule state */
      void setPositions();

      /*! Verify if the active team changed after the last 
       * ballAtFinalPosition. 
       * \return -> true if current active team changed from last turn */
      bool changedTeamToAct();

   protected:
      /*! Get the max number of disk touches per player act
       * \return max number of disk touches per disk act */
      int maxRemainingDiskTouches();

      /*! Get the max number of global touches per act
       * \return max number of global touches per act  */
      int maxRemainingGlobalTouches();

      /*! Verify if the goal that occurred was valid or not */
      void verifyGoalValid();

      /*! Change the current team to act */
      void changeTeamToAct();

      static bool changedBallOwner;  /**< Flag of changed the active team */

      static int state;              /**< Current Rules State */

      static int minutesPerHalf;     /**< Number of minutes per half time */
      static int gameType;           /**< Current Game Type */

      static int halfMinutes;        /**< Current Half Minutes */
      static int halfSeconds;        /**< Current Half Seconds */

      static Uint32 timeStarted;     /**< Time when the current half started */

      static teamPlayer* currentDisk;  /**< Current in play disk */

      static bool secondHalf;        /**< True if is the second half */

      static bool willShoot;         /**< True if the player will try a shoot */

      static int remainingGlobalTouches; /**< Touches to do on play */
      static int remainingDiskTouches;   /**< Remaining consecutive touches */

      static team* teamA;            /**< Current TeamA */
      static team* teamB;            /**< Current TeamB */
      static team* activeTeam;       /**< Current Team in Act */
      static team* upperTeam;        /**< Current team at upper side */
      static team* lastBallCollided; /**< Last team the ball collide to */

      static ball* usedBall;         /**< Current Ball */

      static bool collidedBallFirst; /**< When the player collided ball first */
      static bool collidedOwnDiskFirst; /**< When the player collided with
                                       another disk first of the same player */
      static bool collidedEnemyDiskFirst; /**< When the player collided with another
                                          disk from the other player */

      static int ballAction;        /**< ID of some ball action */
      static float pX;              /**< X coordinate of the action */
      static float pZ;              /**< Z coordinate of the action */
      static bool ballUpper;        /**< True if ball at upper */

};


#endif

