/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>

#ifdef _MSC_VER
   #include <windows.h>
#else
   #include <sys/stat.h>
   #include <sys/types.h>
   #include <errno.h>
#endif

#include <libintl.h>

#include "options.h"
#include "userinfo.h"
#include "defparser.h"

string xutLanguageName[] =
{
   gettext("English"),
   gettext("Portuguese"),
   gettext("Catalan"),
   gettext("Spanish"),
   gettext("Danish"),
   gettext("Italian")
};

string xutLanguageCode[] = 
{
   "en",
   "pt_BR",
   "ca",
   "es",
   "da",
   "it"
};


/***********************************************************************
 *                                 load                                *
 ***********************************************************************/
void options::load()
{
   defParser def;
   string key="", value="";
   int curKey=0;
   string file;
   userInfo info;

   /* First, set the default keys */
   setDefaultKeys();

   /* Get the options filename */
   file = info.getUserHome()+"options.cfg";

   /* Try to load it! */
   if(!def.load(file))
   {
      /* Can't load, so creating the initial */
      cerr << "Can't open options: " << file << endl;
      cerr << "Will create one. " << endl;

      /* Create directories */
#ifdef _MSC_VER
      CreateDirectory(info.getUserHome().c_str(),NULL);
#else
      mkdir(info.getUserHome().c_str(),0755);
#endif

      /* Save the default options */
      save();
      return;
   }
   
   /* Parse all tuples */
   while(def.getNextTuple(key, value))
   {
      if(key == "language")
      {
         sscanf(value.c_str(), "%d", &language);
      }
      else if(key == "antiAliasing")
      {
         sscanf(value.c_str(), "%d", &antiAliasing);
      }
      else if(key == "resolution")
      {
         sscanf(value.c_str(), "%dx%d", &windowX, &windowY);
      }
      else if(key == "fullScreen")
      {
         fullScreen = (value == "true");
      }
      else if(key == "fieldDetail")
      {
         sscanf(value.c_str(), "%d", &fieldDetail);
      }
      else if(key == "soundVolume")
      {
         sscanf(value.c_str(), "%d", &soundVolume);
      }
      else if(key == "cameraType")
      {
         sscanf(value.c_str(), "%d", &cameraType);
      }
      else if(key == "key")
      {
         if(curKey < XUT_TOTAL_KEYS)
         {
            sscanf(value.c_str(), "%d", &keys[curKey]);
            curKey++;
         }
         else
         {
            cerr << "ERROR: Key overflow at options file!" << endl;
         }
      }
      else
      {
         cerr << "ERROR: Unknow option: " << key << "!" << endl;
      }
   }
}

/***********************************************************************
 *                                  save                               *
 ***********************************************************************/
void options::save()
{
   FILE* arq;
   string file;
   userInfo info;
   int i;

   /* Get the options filename */
   file = info.getUserHome()+"options.cfg";

   /* Try to open it */
   if(!(arq = fopen(file.c_str(),"w")))
   {
      cerr << "Error while writing Options: " << file << endl;
      return;
   }

   fprintf(arq, "language = %d\n", language);
   fprintf(arq, "antiAliasing = %d\n", antiAliasing);
   if( (resolutionId == -1) || (!resolutions[resolutionId]) )
   {
      fprintf(arq, "resolution = %dx%d\n", windowX, windowY);
   }
   else
   {
      fprintf(arq, "resolution = %dx%d\n", resolutions[resolutionId]->w, 
                                           resolutions[resolutionId]->h);
   }
   fprintf(arq, "fullScreen = %s\n", fullScreen?"true":"false");
   fprintf(arq, "fieldDetail = %d\n", fieldDetail);
   fprintf(arq, "soundVolume = %d\n", soundVolume);
   fprintf(arq, "cameraType = %d\n", cameraType);

   /* Save All Key-maps */
   for(i=0; i < XUT_TOTAL_KEYS; i++)
   {
      fprintf(arq, "key = %d\n", keys[i]);
   }

   fclose(arq);
}

/***********************************************************************
 *                           defineResolutions                         *
 ***********************************************************************/
void options::defineResolutions()
{
   int i;
   resolutions = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_DOUBLEBUF | 
                                     SDL_OPENGL);

   resolutionId = 0;
   for(i = 0; resolutions[i]; i++)
   {
      if( (resolutions[i]->w == windowX) &&
          (resolutions[i]->h == windowY) )
      {
         resolutionId = i;
      }
   }

}

/***********************************************************************
 *                            setDefaultKeys                           *
 ***********************************************************************/
void options::setDefaultKeys()
{
   keys[XUT_KEY_CAMERA_UP] = SDLK_KP8;
   keys[XUT_KEY_CAMERA_DOWN] = SDLK_KP2;
   keys[XUT_KEY_CAMERA_LEFT] = SDLK_KP4;
   keys[XUT_KEY_CAMERA_RIGHT] = SDLK_KP6;
   keys[XUT_KEY_CAMERA_ROTATE_LEFT] = SDLK_LEFT;
   keys[XUT_KEY_CAMERA_ROTATE_RIGHT] = SDLK_RIGHT;
   keys[XUT_KEY_CAMERA_ROTATE_UP] = SDLK_KP9;
   keys[XUT_KEY_CAMERA_ROTATE_DOWN] = SDLK_KP3;
   keys[XUT_KEY_CAMERA_ZOOM_UP] = SDLK_UP;
   keys[XUT_KEY_CAMERA_ZOOM_DOWN] = SDLK_DOWN;

   keys[XUT_KEY_WILL_SHOOT] = SDLK_s;
   keys[XUT_KEY_REPLAY] = SDLK_r;

   keys[XUT_KEY_ROTATE_LEFT] = SDLK_COMMA;
   keys[XUT_KEY_ROTATE_RIGHT] = SDLK_PERIOD;
}

/***********************************************************************
 *                               getKey                                *
 ***********************************************************************/
int options::getKey(int key)
{
   if( (key >= 0) && (key < XUT_TOTAL_KEYS))
   {
      return(keys[key]);
   }

   return(0);
}

/***********************************************************************
 *                               setKey                                *
 ***********************************************************************/
void options::setKey(int key, int value)
{
   if( (key >= 0) && (key < XUT_TOTAL_KEYS))
   {
      keys[key] = value;
   }
}


/***********************************************************************
 *                            getAntiAliasing                          *
 ***********************************************************************/
int options::getAntiAliasing()
{
   return(antiAliasing);
}

/***********************************************************************
 *                            setAntiAliasing                          *
 ***********************************************************************/
void options::setAntiAliasing()
{ 
   antiAliasing = (antiAliasing + 2) % 6;
}

/***********************************************************************
 *                               getWindowX                            *
 ***********************************************************************/
int options::getWindowX()
{
   return(windowX);
}

/***********************************************************************
 *                               getWindowY                            *
 ***********************************************************************/
int options::getWindowY()
{
   return(windowY);
}

/***********************************************************************
 *                             setWindowSize                           *
 ***********************************************************************/
void options::setWindowSize()
{
   resolutionId++;
   if(!resolutions[resolutionId])
   {
      resolutionId = 0;
   }
}

/***********************************************************************
 *                        getPotentialWindowSize                       *
 ***********************************************************************/
void options::getPotentialWindowSize(int& wX, int& wY)
{
   wX = resolutions[resolutionId]->w;
   wY = resolutions[resolutionId]->h;
}

/***********************************************************************
 *                             getFullScreen                           *
 ***********************************************************************/
bool options::getFullScreen()
{
   return(fullScreen);
}

/***********************************************************************
 *                             setFullScreen                           *
 ***********************************************************************/
void options::setFullScreen()
{
   fullScreen = !fullScreen;
}

/***********************************************************************
 *                             getFieldDetail                          *
 ***********************************************************************/
int options::getFieldDetail()
{
   return(fieldDetail);
}

/***********************************************************************
 *                             setFieldDetail                          *
 ***********************************************************************/
void options::setFieldDetail()
{
   fieldDetail = ((fieldDetail + 1) % (XUT_OPTIONS_FIELD_DETAIL_HIGH+1));
}

/***********************************************************************
 *                             getSoundVolume                          *
 ***********************************************************************/
int options::getSoundVolume()
{
   return(soundVolume);
}

/***********************************************************************
 *                             setSoundVolume                          *
 ***********************************************************************/
void options::setSoundVolume()
{
   if(soundVolume  == 255)
   {
      /* Stop sound */
      soundVolume = 0;
      return;
   }

   /* Sum sound */
   soundVolume = (soundVolume + 10);
   if(soundVolume > 255)
   {
      soundVolume = 255;
   }
}

/***********************************************************************
 *                             getCameraType                           *
 ***********************************************************************/
int options::getCameraType()
{
   return(cameraType);
}

/***********************************************************************
 *                             setCameraType                           *
 ***********************************************************************/
void options::setCameraType()
{
   cameraType = ((cameraType + 1) % (XUT_OPTIONS_CAMERA_TYPE_FOLLOW_BALL+1));
}

/***********************************************************************
 *                              setLanguage                            *
 ***********************************************************************/
void options::setLanguage(int lNumber)
{
   language = lNumber;
   if( (language > XUT_LANGUAGE_IT) || (language < 0) )
   {
      language = XUT_LANGUAGE_EN;
   }
   setLocale();
}

/***********************************************************************
 *                              getLanguage                            *
 ***********************************************************************/
int options::getLanguage()
{
   return(language);
}

/***********************************************************************
 *                            getLanguageName                          *
 ***********************************************************************/
string options::getLanguageName()
{
   return(gettext(xutLanguageName[language].c_str()));
}

/***********************************************************************
 *                               setLocale                             *
 ***********************************************************************/
void options::setLocale()
{
   string saux; 
   /* Set locale */
   setlocale (LC_CTYPE, "");
   setlocale(LC_MESSAGES, "");
   bindtextdomain ("xut", LOCALEDIR);
   textdomain("xut");

   saux = xutLanguageCode[language];

   /* Set language */
#ifdef _MSC_VER
   putenv((string("LANG=" + saux)).c_str());
#else
   setenv("LANGUAGE", saux.c_str(), 1);
   /* Make change known.  */
   {
      extern int  _nl_msg_cat_cntr;
      ++_nl_msg_cat_cntr;
   }
#endif
}

/***********************************************************************
 *                             Static Members                          *
 ***********************************************************************/
int options::keys[XUT_TOTAL_KEYS];
int options::antiAliasing = 4;
int options::windowX = 800;
int options::windowY = 600;
int options::resolutionId = -1;
int options::language = XUT_LANGUAGE_EN;
bool options::fullScreen = false;
int options::fieldDetail = XUT_OPTIONS_FIELD_DETAIL_NORMAL;
int options::soundVolume = 128;
int options::cameraType = XUT_OPTIONS_CAMERA_TYPE_FOLLOW_FOULS;
SDL_Rect** options::resolutions = NULL;

